#!/bin/sh 
#install_patch.sh
echo "Patch MMEV2.2 P01"
echo ""
echo "
This procedure will install a patch to solve IPMT Case HPXL10JTU 
relating to the audio on the AlphaStation 255. The characteristics of 
the problem are that the audio output is unusable, and sounds like 
white noise.
"

#define commands for testing
CP_CMD="cp"
MV_CMD="mv"
RM_CMD="rm"
LN_CMD="ln"
CHOWN_CMD="chown"

LOGFILE=/var/adm/mme_patch.log
NOT_AS255=0

[ `whoami` = "root" ] || {
    echo ""
    echo "Error: Not running as root."
    echo "Please su to root and restart this script."
    exit 1
}

if [ ! -f $LOGFILE ] ; then
    touch $LOGFILE
fi

echo "MME Patch Log Started" `date` >>$LOGFILE

/usr/sbin/uerf -r 300 | grep "AlphaStation 255" >/dev/null
if [ $? != 0 ] ; then
    while :
    do
    echo ""
    echo "Warning: Not running on an AlphaStation 255."
    echo "It is not necessary to install this patch."
    echo "Do you wish to continue? (y/[n]): \c"
    read X
    case "$X" in
    [Yy]*) NOT_AS255=1
	   break
	   ;;
    [Nn]*) exit 1
    esac
    done
fi
echo ""
echo "Please wait while your system is examined..."
echo ""

OSFMAJORVERSION=0
uname -r | grep V3 >/dev/null
if [ $? = 0 ] ; then
    OSFMAJORVERSION=3
fi
uname -r | grep V4 >/dev/null
if [ $? = 0 ] ; then
    OSFMAJORVERSION=4
fi

if [ $OSFMAJORVERSION = 0 ] ; then
    echo "Error: Digital Unix Version not V3.x or V4.x."
    exit 1
fi

MMEINSTALLED=0
MSBINSTALLED=0
RTINSTALLED=0
MMESUBSETS=""
MSBSUBSET=""
MMEVERSION=""

/usr/sbin/setld -i | grep MME | grep installed >/tmp/mmesubsets
if [ $? = 0 ] ; then
    MMEINSTALLED=1
    grep MMERT[12].. /tmp/mmesubsets >/dev/null
    if [ $? = 0 ] ; then
        RTINSTALLED=1 
        MMEVERSION=`what /usr/bin/mmeserver | grep mmeserver \
                    | awk '{ if (NF > 1) print $2 }'`
        MMESUBSET=`grep MMERT[12] /tmp/mmesubsets | awk '{print $1 }'`
    fi
    MSBSUBSET=`grep MSB /tmp/mmesubsets | awk '{print $1 }'`
    if [ X$MSBSUBSET != "X" ] ; then
        MSBINSTALLED=1
    fi
fi

if [ $OSFMAJORVERSION = "4" ]  && [ $MMEINSTALLED = 0 ] ; then
    echo "
Error: Digital UNIX 4.x is installed, but the Multimedia Services
Layered Product is not installed.  First install MMS V2.2, MMS V2.1,
or MMS V2.0A, from either the Digital UNIX Associated Product CD,
or the Digital UNIX Alpha Software Product Library CD.  Be sure to
select the Microsoft Sound Board Driver subset (MMEDRVMSBxxx) for
installation.  During the installation, it is not necessary to
rebuild the kernel.  Following the installation, install this patch,
which will rebuild the kernel with the patched Microsoft Sound Board
driver.
"
fi

if [ $OSFMAJORVERSION = "4" ] && [ $RTINSTALLED = 1 ] \
    && [ $MSBINSTALLED = 0 ] ; then
    echo "

Error: Digital UNIX V4.x is installed, but the Multimedia Services 
Microsoft Sound Board Driver subset (MMEDRVMSBxxx) is not
installed.  First install the MMEDRVMSBxxx subset for MMS $MMEVERSION
from either the Digital UNIX Associated Product CD, or the Digital UNIX
Alpha Software Product Library CD.
During the installation, it is not necessary to rebuild the kernel.
Following the installation, install this patch, which will rebuild
the kernel with the patched Microsoft Sound Board driver.
"
    exit 1
fi

if [ $OSFMAJORVERSION = "4" ] && [ $RTINSTALLED = 0 ] ; then
    echo "
Error: Digital UNIX V4.x is installed, but the Multimedia Services Layered
Product Runtime subset and Microsoft Sound Board Driver subset are
not installed.
First install MMS V2.2, MMS V2.1, or MMS V2.0A, from
either the Digital UNIX Associated Product CD, or the Digital UNIX Alpha
Software Product Library CD.  Be sure to select the Microsoft Sound
Board Driver subset (MMEDRVMSBxxx) for installation.  During the
installation, it is not necessary to rebuild the kernel.  Following
the installation, install this patch, which will rebuild the kernel
with the patched Microsoft Sound Board driver.
"
    exit 1
fi


########################################################################
echo "*************************************************************"
echo "The following information has been extracted from your system:"
echo ""
if [ $NOT_AS255 = 1 ] ; then
    echo "This system does not appear to be an AlphaStation 255."
fi
echo "Digital UNIX V${OSFMAJORVERSION}.x installed."
if [ $MMEINSTALLED = 1 ] ; then
    echo "Multimedia Services $MMEVERSION installed."
else
    echo "Multimedia Services NOT installed."
fi
if [ $RTINSTALLED = 1 ] ; then
    echo "MMS Runtime installed."
elif [ $MMEINSTALLED = 1 ] ; then
    echo "MMS Runtime NOT installed."
fi
if [ $MSBINSTALLED = 1 ] ; then
    echo "MMS Microsoft Sound Board Driver installed."
elif [ $MMEINSTALLED = 1 ] ; then
    echo "MMS Microsoft Sound Board Driver NOT installed."
fi

if [ $MMEINSTALLED = 1 ] && [ $MMESUBSET != "MMERT220" ] \
    && [ $MMESUBSET != "MMERT210" ] && [ $MMESUBSET != "MMERT201" ] \
    && [ $MMESUBSET != "MMERT200" ] && [ $MMESUBSET != "MMERT160" ] ; then
    echo "
Warning: A version of Multimedia Services is installed which is
not recognized or not supported for this patch. 
"
fi
 

if [ $OSFMAJORVERSION = "3" ] && [ $RTINSTALLED = 1 ] \
   && [ $MSBINSTALLED = 0 ]; then
    echo "
Digital UNIX V3.x, and the Multimedia Services Layered
Product Runtime subset is installed, but the Microsoft Sound Board Driver
subset is not installed.

It is recommended that you first install the Multimedia Services
$MMEVERSION Microsoft Sound Board driver, and then install this patch.
"
fi

if [ $MSBINSTALLED = 1 ] && [ $MSBSUBSET != "MMEDRVMSB220" ] \
    && [ $MSBSUBSET != "MMEDRVMSB210" ] && [ $MSBSUBSET != "MMEDRVMSB201" ] \
    && [ $MSBSUBSET != "MMEDRVMSB200" ] && [ $MSBSUBSET != "MMEMSB160" ] ; then
    echo "
Warning: A version of Multimedia Services is installed which is
not recognized or not supported for this patch. 
"
fi


while :
do
    echo "Continue with installation of this patch? (y/n): \c"
    read X
    case "$X" in
    [Nn]*) echo "Exiting"
	   exit 1
           ;;
    [Yy]*) break
           ;;
    esac
done

echo ""
echo "Continuing..."
echo ""
# What has to be done:
# If DU V4.x, [ MSB driver subset must be installed ]
#     1. Replace the msb driver[s] in the subset.
#     2. Replace the hard link at /sys/BINARY/msb.mod.
# If DU V3.x and the MSB driver subset are installed,
#     1. Replace the two versions of the msb driver in the subset.
# If DU V3.x and MSB driver subset is not installed,
#     1. Replace the driver in /sys/BINARY.

# Remove hard link on DU V4.x
if [ $OSFMAJORVERSION = "4" ] ; then
    echo "$RM_CMD /sys/BINARY/msb.mod" >> $LOGFILE
    $RM_CMD /sys/BINARY/msb.mod 
fi

if [ $MSBINSTALLED = 1 ] ; then 
# determine kit locations of driver objects
    MSBV3_LOC=`grep msb.o /usr/.smdb./$MSBSUBSET.inv | awk '{ print $10 }'`
    MSBV4_LOC=`grep msb.mod /usr/.smdb./$MSBSUBSET.inv | awk '{ print $10 }'`

# move previous driver objects out of the way
    if [ -f /$MSBV3_LOC ] && [ ! -f /$MSBV3_LOC.orig# ] ; then
        echo "$MV_CMD /$MSBV3_LOC /$MSBV3_LOC.orig#" >>$LOGFILE
        $MV_CMD /$MSBV3_LOC /$MSBV3_LOC.orig#
    fi
    if  [ -f /$MSBV4_LOC ] && [ ! -f /$MSBV4_LOC.orig# ] ; then
        echo "$MV_CMD /$MSBV4_LOC /$MSBV4_LOC.orig#" >> $LOGFILE
        $MV_CMD /$MSBV4_LOC /$MSBV4_LOC.orig#
    fi
# install new patched objects into kit subset locations
    if [ X$MSBV3_LOC != "X" ] ; then
        echo "$CP_CMD msb.o.v3 /$MSBV3_LOC" >> $LOGFILE
        $CP_CMD msb.o.v3 /$MSBV3_LOC ; $CHOWN_CMD bin:bin /$MSBV3_LOC
    fi
    if [ X$MSBV4_LOC != "X" ] ; then
        echo "$CP_CMD msb.mod.v4 /$MSBV4_LOC" >> $LOGFILE
        $CP_CMD msb.mod.v4 /$MSBV4_LOC ; $CHOWN_CMD bin:bin /$MSBV4_LOC
    fi
fi

if [ $OSFMAJORVERSION = "4" ] ; then
# link msb.mod file to /sys/BINARY/.
    echo "$LN_CMD /$MSBV4_LOC /sys/BINARY/." >> $LOGFILE
    $LN_CMD /$MSBV4_LOC /sys/BINARY/.
fi

if [ $OSFMAJORVERSION = "3" ] && [ $MSBINSTALLED = 0 ] ; then
    if [ -f /sys/BINARY/msb.o ] && [ ! -f /sys/BINARY/msb.o.orig# ] ; then
        echo "$MV_CMD /sys/BINARY/msb.o /sys/BINARY/msb.o.orig#" >> $LOGFILE
        $MV_CMD /sys/BINARY/msb.o /sys/BINARY/msb.o.orig# 
    fi
    echo "$CP_CMD msb.o.v3 /sys/BINARY/msb.o" >> $LOGFILE
    $CP_CMD msb.o.v3 /sys/BINARY/msb.o ; $CHOWN_CMD bin:bin /sys/BINARY/msb.o
fi

if [ $OSFMAJORVERSION = "3" ] && [ $MSBINSTALLED = 1 ] ; then
    echo "$CP_CMD msb.o.v3 /sys/BINARY/msb.o" >> $LOGFILE
    $CP_CMD msb.o.v3 /sys/BINARY/msb.o ; $CHOWN_CMD bin:bin /sys/BINARY/msb.o
fi

echo "MME Patch Log Ended `date`" >>$LOGFILE
echo "**************************************" >>$LOGFILE
echo "**************************************"
echo "The MMEV2.2 P01 patch has been installed."
echo "
The file /var/adm/mme_patch.log contains the shell commands that
were executed in order to install the patch.
"
HOSTNAME=`/bin/hostname`
[ "$HOSTNAME" ] &&
{
    HOSTNAME=`expr "$HOSTNAME" : '\([a-zA-Z0-9]*\)'`
    HOSTNAME=`echo "$HOSTNAME" | dd conv=ucase 2>/dev/null`
}
NOCONFIGFILE=0
if [ ! -f /sys/conf/$HOSTNAME ] ; then
    NOCONFIGFILE=1
else
    grep msb0 /sys/conf/$HOSTNAME | grep controller >/dev/null
    NOMSBLINE=$?
fi
INVOKE_BUILD=0
echo "In order to make the change effective, the kernel must be rebuilt."
if [ $NOCONFIGFILE = 0 ] ; then

    if [ $MSBINSTALLED = 1 ] ; then
        INVOKE_BUILD=1
        echo "The following command will be invoked to rebuild the kernel:"
	echo ""
        BUILD_CMD="/usr/sbin/setld -c $MSBSUBSET INSTALL"
        echo "$BUILD_CMD"
        echo "
Please respond \"y\" to the 'Do you wish to install the Microsoft
Sound Board option module device driver' question, even though
the device driver is already installed.
"
    elif [ $NOMSBLINE = 0 ] ; then
        echo "The following command will be invoked to rebuild the kernel:"
        INVOKE_BUILD=1
        BUILD_CMD="doconfig -c $HOSTNAME"
        echo "$BUILD_CMD"
        echo "
If $HOSTNAME is not the configuration file you wish to use for rebuilding
the kernel, then respond \"N\" and manually run doconfig.
"
    else 
        echo "
However, the kernel config file /sys/conf/$HOSTNAME does not contain a line
of the form \"msb0 controller ...\".  It is recommended that you boot
genvmunix and run doconfig.  You should specify the option to create a new
kernel configuration file.
"
    fi
else
    echo "
There is no /sys/conf/$HOSTNAME file. Run doconfig manually in order to
build a new kernel.  If you are going to specify the option to create a new
kernel configuration file, it is recommended that you boot genvmunix and run
doconfig.
"
fi

echo "Following the kernel rebuild you must move the new kernel to /vmunix"
echo "and then boot the new kernel."
while [ INVOKE_BUILD = 1 ]
    do 
    echo ""
    echo "Do you want to rebuild the kernel now? (y/n) \c"
    read X
    case "$X" in
    [Yy]*) break
	   ;;
    [Nn]*) echo "exiting"
	   exit 0
    esac
    done

if [ $INVOKE_BUILD = 1 ] ; then
    $BUILD_CMD
fi

echo "End of MMS V2.2 P01 patch script."
